import {
  select,
  csv,
  extent,
  axisBottom,
  scaleOrdinal,
  event
} from 'd3';

import { loadData } from './loadData';
import { dropdownMenu } from './dropdownMenu';
import { scatterPlot } from './scatterPlot';
import { stemLegend } from './stemLegend';
import { colorLegend } from './colorLegend';
import { table, updateTable } from './table';

// create color value
const colorScheme = [
  '#373D75', '#50579E',
  '#6971C9', '#9BA0D9', 
  '#D4A2D3', '#9A5791', 
  '#734571', '#A24F4D', 
  '#DC9A9D', '#E3CB9B', 
  '#E1BA64', '#B99E4B', 
  '#886E3B', '#687841', 
  '#91A05C', '#D1D9A3'];

let xColumn;
let yColumn;
let columns;
let input;
let category1;
let category2;

let selectedCate2;
let selectedCate1;


loadData().then(data => {
  input = data;
  category1 = [...new Set(input.map(data => data.Major_category))];
  category2 = [...new Set(input.map(data => data.stem))];
  xColumn = data.columns[5];
  columns = data.columns;
  
  select('#grid').call(table, {
    data: input,
    columns: columns.filter(c => 
      c === 'Total' ||
      c === 'ShareWomen' ||
      c === 'Unemployment_rate' ||
      c === 'Median ($)' ||
      c === 'Major'
    )
  });
  render();
});

const svg = select('svg');
const body = select('body');
const stemLegendG = svg.append('g')
  .attr('transform', 'translate(480, 600)');
const colorLegendG = svg.append('g')
  .attr('transform', 'translate(570, 45)');


const width = +svg.attr('width');
const height = +svg.attr('height');

const colorValue1 = d => d.Major_category;
const colorValue2 = d => d.stem;
const colorScale1 = scaleOrdinal();
const colorScale2 = scaleOrdinal();

const onClick2 = d => {
  selectedCate2 = d;
  render();
};

const onClick1 = d => {
  selectedCate1 = d;
  update(filterByCategory(selectedCate1));
  render();
};

var filterByCategory = category => {
  return input.filter(d => d.Major_category === category);
};

const onXColumnClicked = column => {
  xColumn = column;
  render();
};

// mouseover and out event handler
let tipMouseOver = function(d) {
  let html = d.Major;
  select('.tooltip').html(html)
    .style('left', event.pageX +15 + 'px')
    .style('top', event.pageY - 50 + 'px')
    .transition()
    .duration(200)
    .style('opacity', 0.8);
};

let tipMouseOut = function (d) {
  select('.tooltip').transition()
    .duration(200)
    .style("opacity", 0);
};

const update = data => {
  if(data === null || data.length === 0) 
    data = input;
  select('#grid table').call(updateTable, {
    data, 
    columns: columns.filter(c => 
      c === 'Total' ||
      c === 'ShareWomen' ||
      c === 'Unemployment_rate' ||
      c === 'Median ($)' ||
      c === 'Major'
    )
  });
};

const render = () => {
  
  select('#x-menus')
    .call(dropdownMenu, {
      options: input.columns.filter(c => 
        c !== 'Major' &&
        c !== 'Major_category' &&
        c !== 'stem'
      ),
      onOptionClicked: onXColumnClicked,
      selectedOption: xColumn
    });
  
  colorScale1
    .domain(category1)
    .range(colorScheme);
  
  colorScale2
    .domain(category2)
    .range(2);
  
  svg.call(scatterPlot, {
    yValue: d => d.Major_category,
    //xAxisLabel: xColumn,
    xValue: d => d[xColumn],
    //yAxisLabel: yColumn,
    circleRadius: 7,
    margin: { top: 20, right: 390, bottom: 50, left: 30 },
    width,
    height,
    data: input,
    colorScale1,
    colorValue1,
    selectedCate1,
    selectedCate2,
    tipMouseOver,
    tipMouseOut
  });
  
  stemLegendG.call(stemLegend, {
    circleRadius: 5,
    colorScale2,
    colorValue2,
    selectedCate2,
    onClick2
  });
  
  colorLegendG.call(colorLegend, {
    circleRadius: 8,
    colorScale1,
    colorValue1,
    selectedCate1,
    onClick1
  });
  
  select('body').append("div")
    .attr("class", "tooltip")
    .style("opacity", 0);
  
};

