import {
  select,
  csv,
  scaleLinear,
  extent,
  axisRight,
  axisBottom,
  format,
  scalePoint,
  //scaleLog
} from 'd3';

export const scatterPlot = (selection, props) => {
  const {
    xValue,
    xAxisLabel,
    yValue,
    circleRadius,
    yAxisLabel,
    margin,
    width,
    height,
    data,
    colorScale1,
    colorValue1,
    selectedCate1,
    selectedCate2,
    tipMouseOver,
    tipMouseOut
  } = props;
  
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  
  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([0, innerWidth]);
  
  const yScale = scalePoint()
    .domain(data.map(yValue))
    .range([0, innerHeight])
    .padding(1);
  
  const g = selection.selectAll('.container').data([null]);
  const gEnter = g
    .enter().append('g')
      .attr('class', 'container');
  gEnter
    .merge(g)
    .attr('transform', `translate(${margin.left},${margin.top})`);
  
  const AxisTickFormat = number => 
    (number > 1) 
      ? format('.2s')(number)
      : number = number;
  
  const xAxis = axisBottom(xScale)
    .tickFormat(AxisTickFormat)
    .tickSize(-innerHeight)
    .tickPadding(15);
    //.tickSizeOuter();
  
  const yAxis = axisRight(yScale)
    .tickFormat(AxisTickFormat)
    .tickSize(innerWidth)
    .tickPadding(10);
    //.tickSizeOuter(1);
  
  const yAxisG = g.select('.y-axis');
  const yAxisGEnter = gEnter
    .append('g')
      .attr('class', 'y-axis');
  
  yAxisG
    .merge(yAxisGEnter)
      .call(yAxis).selectAll('.domain')
        .attr('stroke', '#C0C0BB');
  yAxisG
    .merge(yAxisGEnter)
      .call(yAxis).selectAll('text').remove();

  /*
  const yAxisLabelText = yAxisGEnter
    .append('text')
      .attr('class', 'axis-label')
      .attr('y', -50)
    .merge(yAxisG.select('.axis-label'))
      .attr('x', -innerHeight / 2)
      .text(yAxisLabel);
  */
  
  const xAxisG = g.select('.x-axis');
  const xAxisGEnter = gEnter
    .append('g')
      .attr('class', 'x-axis');
  
  xAxisG
    .merge(xAxisGEnter)
      .attr('transform', `translate(0,${innerHeight})`)
      .call(xAxis)
      .selectAll('.domain')
        .attr('stroke', '#C0C0BB');
  
  const xAxisLabelText = xAxisGEnter
  .append('text')
    .attr('class', 'axis-label')
    .attr('y', 75)
    //.attr('fill', 'black')
  .merge(xAxisG.select('.axis-label'))
    .attr('x', innerWidth / 2)
    .text(xAxisLabel);
  
  /*
  xAxisGEnter.selectAll('text')
    .attr("transform", "rotate(-45)")
    .style("text-anchor", "front");
  */
  
  const circles = g.merge(gEnter)
    .selectAll('circle').data(data);
  circles
    .enter().append('circle')
      .attr('cx', 0)
      .attr('cy', innerHeight/2)
      //.attr('fill','grey')
      .attr('fill', d => colorScale1(colorValue1(d)))
      .attr('r', 0)
      .on('mouseover', tipMouseOver)
      .on('mouseout', tipMouseOut)
    .merge(circles)
    .transition().duration(500)
    //.delay((d, i) => i * 5)
      .attr('cy', d => (yScale(yValue(d)) + 15 - Math.random() * 30))
      .attr('cx', d => xScale(xValue(d)))
      .attr('r', circleRadius)
      .attr('opacity', d => 
        (!selectedCate1 || selectedCate1 === d.Major_category)
        ? 0.7
        : 0.05)
      
      .attr('fill', d => colorScale1(colorValue1(d)))
        // (selectedCate1 === d.Major_category)
        // ?  colorScale1(colorValue1(d))
        // : 'grey')
      
      .attr('stroke-width', 3)
      .attr('stroke', d => 
        (selectedCate2 === d.stem)
        ? '#26c3db'
        : null);
};